/* Scanmap is a 3d Visualisation for Snort alerts 
*   
*   Copyright (C) 2003 Daniel Clark
*
*	This program is free software; you can redistribute it and/or 
*	modify it under the terms of the GNU General Public License as 
*	published by the Free Software Foundation; either version 2 of 
*	the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful, 
*	but WITHOUT ANY WARRANTY; without even the implied warranty of 
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License 
*	along with this program; if not, write to the Free Software 
*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*	
*	Contact: Daniel Clark 
*	Email:	scanmap3d@sourceforge.net
*	
*
*
* animpacket3d.java
*
* Created on March 14, 2003, 12:28 PM
*/

package net.ossim.scanmap;

import java.lang.Math.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Text2D;
import java.util.*;
import com.sun.j3d.utils.behaviors.vp.*;
import com.sun.j3d.utils.behaviors.interpolators.*;
import com.sun.j3d.utils.timer.J3DTimer;
import java.awt.*;

public class animpacket3d extends javax.media.j3d.TransformGroup {
    
    public Alpha animAlpha;
    public host3d host1,host2;
    public int port1,port2;
    public float width;
    public Appearance app;
    private net3dapp textures;
    public KBRotPosScaleSplinePathInterpolator linearInterpolator;
    public long alphatime;
    
    public animpacket3d(net3dapp texture_in) {
        textures = texture_in;
    }
    
    
    public Appearance getPacketAppearance(int packet_type){

	switch (packet_type){
		case	6:{
		        return textures.getAppearance(10);	
			}
		case    17:{
			return textures.getAppearance(11);
			}
		case 	1:{
			return textures.getAppearance(12);
			}
		default:{
		return textures.getAppearance(13);
		}
	}

    }
    
    /** Creates a new packet */
    public animpacket3d(host3d inhost2, host3d inhost1, ip_packet inpacket,net3dapp in_app)
    {        
        
        textures = in_app;
        this.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
            
        host1=inhost1;
        host2=inhost2;
        port1=inpacket.src_port;
        port2=inpacket.dst_port;
        
        app = getPacketAppearance(inpacket.protocol);        
        
        port3d port13d = new port3d();
        port3d port23d = new port3d();
        TransformGroup port1tg = new TransformGroup();
        TransformGroup port2tg = new TransformGroup();
        TransformGroup host1g = new TransformGroup();
        TransformGroup host2g = new TransformGroup();
        TransformGroup pgroup = new TransformGroup();
        host1g = (TransformGroup) host1.getChild(0);
        host2g = (TransformGroup) host2.getChild(0);
        Transform3D hosttrans1 = new Transform3D();
        Transform3D hosttrans2 = new Transform3D();
        Transform3D hosttrans1g = new Transform3D();
        Transform3D hosttrans2g = new Transform3D();       
        Transform3D porttrans1 = new Transform3D();
        Transform3D porttrans2 = new Transform3D();
        Vector3f hostvector1 = new Vector3f();
        Vector3f hostvector2 = new Vector3f();
        Vector3f portvector1 = new Vector3f();
        Vector3f portvector2 = new Vector3f();
        Vector3f nullvector = new Vector3f(0.0f,0.0f,0.0f);
        Vector3f linkvector1 = new Vector3f();
        Vector3f linkvector2 = new Vector3f();
        host1.getTransform(hosttrans1);
        host2.getTransform(hosttrans2);                                
        host1g.getTransform(hosttrans1g);
        host2g.getTransform(hosttrans2g);
        hosttrans1.get(hostvector1);
        hosttrans2.get(hostvector2);  
        
        
        // draw icmp from host to host
        if (inpacket.protocol == 1){
            hosttrans1.mul(hosttrans1g);
            hosttrans2.mul(hosttrans2g);
            hosttrans1.get(hostvector1);
            hosttrans2.get(hostvector2);             
            linkvector1 = hostvector1;
            linkvector2 = hostvector2;
            
        }
        // not an icmp packet
        else {            
            port13d = (port3d) host1.getPort(port1);
            port23d = (port3d) host2.getPort(port2);                
            port1tg = (TransformGroup) port13d.getChild(0);
            port2tg = (TransformGroup) port23d.getChild(0);        
            port1tg.getTransform(porttrans1);
            port2tg.getTransform(porttrans2);
            porttrans1.get(portvector1);
            porttrans2.get(portvector2);
            hosttrans1.mul(porttrans1);
            hosttrans2.mul(porttrans2);
            hosttrans1.get(linkvector1);
            hosttrans2.get(linkvector2);    
        }
        
        pgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        pgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        
        float size = (float) inpacket.size / 1500.0f * 1.0f +0.2f;
        
        
        switch (inpacket.protocol) {
            case 6:
            {
                Sphere newpacket = new Sphere(size,Sphere.GENERATE_TEXTURE_COORDS,4,app);
                
                
                if (inpacket.flags!=null){
                    
                    // New 2D Text
					TransformGroup scale_text = new TransformGroup();
        			Transform3D scale_trans = new Transform3D();
        			scale_trans.setScale(10);
                    String flagString = inpacket.flags;    
                    float flaglength = flagString.length();
                    TransformGroup offset = new TransformGroup();
                    Transform3D text_offset = new Transform3D();
                    text_offset.setTranslation(new Vector3f(0.0f,-2.0f,1.0f));

		        if (flagString.length()>24){
			  	flagString=flagString.substring(0,20);	
		        }
                    Text2D flagname2D = new Text2D(flagString,new Color3f(0.0f,1.0f,0.0f),"Serif",12,Font.PLAIN);
                    OrientedShape3D textShape = new OrientedShape3D(flagname2D.getGeometry(), 
                        flagname2D.getAppearance(),OrientedShape3D.ROTATE_ABOUT_POINT, 
                        new Point3f(0.0f,0.0f,0.0f));                
					scale_text.setTransform(scale_trans);  
					scale_text.addChild(textShape);					             
                    offset.addChild(scale_text);  
                    offset.setTransform(text_offset);     
                    pgroup.addChild(offset);

                }
                newpacket.setPickable(false);
                pgroup.addChild(newpacket);
                break;
            }
            case 1:                
            {
                
                Box newpacket = new Box(size,size,size,Box.GENERATE_TEXTURE_COORDS,app);
                if (inpacket.flags!=null){
                    
                    
                    // New 2D Text
					TransformGroup scale_text = new TransformGroup();
        			Transform3D scale_trans = new Transform3D();
        			scale_trans.setScale(10);
                    String flagString = inpacket.flags;    
                    float flaglength = flagString.length();
                    TransformGroup offset = new TransformGroup();
                    Transform3D text_offset = new Transform3D();
                    text_offset.setTranslation(new Vector3f(0.0f,-2.0f,1.0f));
	
                    Text2D flagname2D = new Text2D(flagString,new Color3f(0.0f,1.0f,0.0f),"Serif",12,Font.PLAIN);
                    OrientedShape3D textShape = new OrientedShape3D(flagname2D.getGeometry(), 
                        flagname2D.getAppearance(),OrientedShape3D.ROTATE_ABOUT_POINT, 
                        new Point3f(0.0f,0.0f,0.0f));
					scale_text.setTransform(scale_trans);  
					scale_text.addChild(textShape);					             
                    offset.addChild(scale_text);  
                    offset.setTransform(text_offset);      
                    pgroup.addChild(offset);
                                                           
                }
                newpacket.setUserData("ignore");
                newpacket.setPickable(false);
                pgroup.addChild(newpacket);

                break;
            }
            case 17:
            {
                Cylinder newpacket = new Cylinder(size,size,Cylinder.GENERATE_TEXTURE_COORDS,app);
                newpacket.setPickable(false);
                newpacket.setUserData("ignore");
                pgroup.addChild(newpacket);
                break;
            }
            
            default:
                Cone newpacket = new Cone(size,size,Cone.GENERATE_TEXTURE_COORDS,app);
                newpacket.setPickable(true);
                newpacket.setUserData("ignore");
                pgroup.addChild(newpacket);
            
        }
        

        
        BoundingSphere bounds =
	    new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
        
        KBKeyFrame[] linearKeyFrames = new KBKeyFrame[2];
        
        Transform3D t3dKnot = new Transform3D();
        
        Point3f p = new Point3f (linkvector1);
        
        float head  = 0.0f;                          // heading
        float pitch = 0.0f;                          // pitch 
        float bank  = 0.0f;                          // bank 
        Point3f s = new Point3f(1.0f, 1.0f, 1.0f);
      
        linearKeyFrames[0] = 
            new KBKeyFrame(0.0f, 1, p, head, pitch, bank, s, 0.0f, 0.0f, 0.0f); 
        
        p = new Point3f (linkvector2);    
        linearKeyFrames[1] = 
            new KBKeyFrame(1.0f, 1, p, head, pitch, bank, s, 0.0f, 0.0f, 0.0f); 
      
        
        Transform3D yAxis = new Transform3D();
        
        alphatime = (long) ((J3DTimer.getValue()- scanmap3d.starttime)/1000000.0);
        animAlpha = new Alpha (1,Alpha.INCREASING_ENABLE,
            alphatime+scanmap3d.delay,0,scanmap3d.speed,0,0,0,0,0);

        linearInterpolator =
         new KBRotPosScaleSplinePathInterpolator(animAlpha, pgroup,
                                                  yAxis, linearKeyFrames);         
        linearInterpolator.setSchedulingBounds(bounds);
                
        pgroup.addChild(linearInterpolator);      
        

        BranchGroup pbranch = new BranchGroup();
        pbranch.setCapability(BranchGroup.ALLOW_DETACH);
        pbranch.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
        pbranch.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        pbranch.addChild(pgroup);               
        pbranch.compile();
        this.addChild(pbranch);   
        
        
    }
    
    public void updatePacket() {
                        
        port3d port13d = new port3d();
        port3d port23d = new port3d();
        TransformGroup port1tg = new TransformGroup();
        TransformGroup port2tg = new TransformGroup();
        TransformGroup host1g = new TransformGroup();
        TransformGroup host2g = new TransformGroup();
        host1g = (TransformGroup) host1.getChild(0);
        host2g = (TransformGroup) host2.getChild(0);
        Transform3D hosttrans1 = new Transform3D();
        Transform3D hosttrans2 = new Transform3D();
        Transform3D hosttrans1g = new Transform3D();
        Transform3D hosttrans2g = new Transform3D();       
        Transform3D porttrans1 = new Transform3D();
        Transform3D porttrans2 = new Transform3D();
        Vector3f hostvector1 = new Vector3f();
        Vector3f hostvector2 = new Vector3f();
        Vector3f portvector1 = new Vector3f();
        Vector3f portvector2 = new Vector3f();
        Vector3f nullvector = new Vector3f(0.0f,0.0f,0.0f);
        Vector3f linkvector1 = new Vector3f();
        Vector3f linkvector2 = new Vector3f();
        host1.getTransform(hosttrans1);
        host2.getTransform(hosttrans2);                                
        host1g.getTransform(hosttrans1g);
        host2g.getTransform(hosttrans2g);
        hosttrans1.get(hostvector1);
        hosttrans2.get(hostvector2);               

        if (this.port1 == -1||this.port2==-1){
            hosttrans1.mul(hosttrans1g);
            hosttrans2.mul(hosttrans2g);
            hosttrans1.get(hostvector1);
            hosttrans2.get(hostvector2);             
            linkvector1 = hostvector1;
            linkvector2 = hostvector2;
            
        }        

        else {            
            port13d = (port3d) host1.getPort(port1);
            port23d = (port3d) host2.getPort(port2);                
            port1tg = (TransformGroup) port13d.getChild(0);
            port2tg = (TransformGroup) port23d.getChild(0);        
            port1tg.getTransform(porttrans1);
            port2tg.getTransform(porttrans2);
            porttrans1.get(portvector1);
            porttrans2.get(portvector2);
            hosttrans1.mul(porttrans1);
            hosttrans2.mul(porttrans2);
            hosttrans1.get(linkvector1);
            hosttrans2.get(linkvector2);    
        }        
        
        TransformGroup pgroup = new TransformGroup();
        BranchGroup pbranch = new BranchGroup();
        pbranch = (BranchGroup) this.getChild(0);
        pgroup = (TransformGroup) pbranch.getChild(0);
                BoundingSphere bounds =
            new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
                
        KBKeyFrame[] linearKeyFrames = new KBKeyFrame[2];
        Transform3D t3dKnot = new Transform3D();
        Point3f p = new Point3f (linkvector1);
        float head  = 0.0f;                          // heading
        float pitch = 0.0f;                          // pitch 
        float bank  = 0.0f;                          // bank 
        Point3f s = new Point3f(1.0f, 1.0f, 1.0f);
        linearKeyFrames[0] = 
            new KBKeyFrame(0.0f, 1, p, head, pitch, bank, s, 0.0f, 0.0f, 0.0f); 
        p = new Point3f (linkvector2);    
        linearKeyFrames[1] = 
            new KBKeyFrame(1.0f, 1, p, head, pitch, bank, s, 0.0f, 0.0f, 0.0f); 
        Transform3D yAxis = new Transform3D();            
        
        KBRotPosScaleSplinePathInterpolator linearInterpolator2;
        linearInterpolator2 = (KBRotPosScaleSplinePathInterpolator)
	    this.linearInterpolator;             
        linearInterpolator2.setTarget(pgroup);
        linearInterpolator2.setKeyFrames(linearKeyFrames);                     
                                                                          
        
    }    
    
    
}
