/* Scanmap is a 3d Visualisation for Snort alerts 
*   
*   Copyright (C) 2003 Daniel Clark
*
*	This program is free software; you can redistribute it and/or 
*	modify it under the terms of the GNU General Public License as 
*	published by the Free Software Foundation; either version 2 of 
*	the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful, 
*	but WITHOUT ANY WARRANTY; without even the implied warranty of 
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License 
*	along with this program; if not, write to the Free Software 
*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*	
*	Contact: Daniel Clark 
*	Email:	scanmap3d@sourceforge.net
*	
*
*
* host3d.java 
*
* Created on March 14, 2003, 12:28 PM
*/

package net.ossim.scanmap;

import java.lang.Math.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.geometry.Text2D;
import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.image.*;
import java.awt.image.*;
import java.util.*;
import java.awt.*;
import com.sun.j3d.utils.picking.*;
import java.net.*;


public class host3d extends javax.media.j3d.TransformGroup {
    
    public String host_ip;
    public int ports_list_length;
    public boolean selected;
    public boolean details;
    public int net_type;
    private net3dapp app;   
    
    public host3d(net3dapp in_app) {
        app=in_app;
    }
    
    /** Creates a new instance of host3d */
    public host3d(String ip,net3dapp in_app,scanmap3d in_scanmap,int in_type) {
        app=in_app;
        
        this.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.setPickable(true);
        String hostname;
        host_ip = ip;
        hostname="none";
	net_type = in_type;
        
        // DNS Resolution        
        if (in_scanmap.dnslookup){            
            try {
                    InetAddress ip_address = InetAddress.getByName(ip);
                    hostname = ip_address.getHostName();
            } catch(Exception e){
                    hostname = "none";
            }
        }
                
        ports_list_length=0;      
        selected=false;     
        details=false;
        
        TransformGroup hostgroup = new TransformGroup();
        Transform3D hosttransform = new Transform3D();
        hosttransform.set(new Vector3f(0.0f,0.0f,2.0f));
        
        hostgroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        hostgroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        hostgroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        hostgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);      
        hostgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        hostgroup.setPickable(true);
        hostgroup.setTransform(hosttransform);                       
        
        // Add the main host
        Sphere newsphere = new Sphere(1.0f,Sphere.GENERATE_TEXTURE_COORDS,6,app.getAppearance(20+net_type));
	
        newsphere.setCapability(Sphere.ENABLE_PICK_REPORTING);
        newsphere.setCapability(Sphere.ENABLE_GEOMETRY_PICKING);
        newsphere.setCapability(Sphere.ALLOW_BOUNDS_READ);
        newsphere.setCapability(Sphere.ALLOW_BOUNDS_READ);
        newsphere.setCapability(Sphere.ALLOW_CHILDREN_READ);
        newsphere.setPickable(true);
        
                      
        newsphere.setUserData(ip);                
        Sphere selectsphere = new Sphere(1.5f,Sphere.GENERATE_TEXTURE_COORDS,6,app.getAppearance(4));
        selectsphere.setPickable(false);
        Switch select_switch = new Switch();
        select_switch.addChild(selectsphere);
        select_switch.setCapability(Switch.ALLOW_SWITCH_WRITE);
        select_switch.setWhichChild(Switch.CHILD_NONE);
        
        hostgroup.addChild(newsphere);
        hostgroup.addChild(select_switch); 
        
        // Add a label
        
        float ipl = host_ip.length();
        
        // New 2D Text
        
        TransformGroup offset = new TransformGroup();
        TransformGroup scale_text = new TransformGroup();
        Transform3D scale_trans = new Transform3D();
        scale_trans.setScale(10);
        Transform3D text_offset = new Transform3D();
        text_offset.setTranslation(new Vector3f(0.0f,-2.0f,1.0f));                   
        Text2D hostip2D = new Text2D(host_ip,new Color3f(0.0f,1.0f,0.5f),"Serif",10,Font.PLAIN);   
        OrientedShape3D textShape = new OrientedShape3D(hostip2D.getGeometry(), 
            hostip2D.getAppearance(),OrientedShape3D.ROTATE_ABOUT_POINT, 
            new Point3f( 0.0f, 0.0f, 0.0f)); 
        scale_text.setTransform(scale_trans);
        scale_text.addChild(textShape);
        offset.addChild(scale_text);
        offset.setTransform(text_offset);
        hostgroup.addChild(offset);
        
        // add a DNS Name
        if (!hostname.equals("none")&&!hostname.equals(host_ip)){

            TransformGroup offset2 = new TransformGroup();
            Transform3D text_offset2 = new Transform3D();
            TransformGroup scale_text2 = new TransformGroup();
            Transform3D scale_trans2 = new Transform3D();
            scale_trans2.setScale(10);
            text_offset2.setTranslation(new Vector3f(0.0f,-2.0f,1.5f));
             
            Text2D hostname2D = new Text2D(hostname,new Color3f(0.0f,1.0f,0.8f),"Serif",10,Font.PLAIN);   
            OrientedShape3D textShape2 = new OrientedShape3D(hostname2D.getGeometry(), 
                hostname2D.getAppearance(),OrientedShape3D.ROTATE_ABOUT_POINT, 
                new Point3f( 0.0f, 0.0f, 0.0f));         
            scale_text2.setTransform(scale_trans2);
            scale_text2.addChild(textShape2);
            offset2.addChild(scale_text2);              
            offset2.setTransform(text_offset2);         
            hostgroup.addChild(offset2);
        }
        
        this.addChild(hostgroup);        
                
        TransformGroup portgroup = new TransformGroup();        
        portgroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        portgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        portgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        portgroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        portgroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        portgroup.setPickable(true);
        
        TransformGroup infogroup = new TransformGroup();
        infogroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        infogroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        infogroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        infogroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);      
        infogroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        infogroup.setPickable(true);
        Cone infocone = new Cone(1.0f,1.0f,6,6,Cone.GENERATE_TEXTURE_COORDS,app.getAppearance(5));
        infocone.setCapability(Cone.ENABLE_PICK_REPORTING);
        infocone.setCapability(Cone.ENABLE_GEOMETRY_PICKING);
        infocone.setCapability(Cone.ALLOW_BOUNDS_READ);
        infocone.setCapability(Cone.ALLOW_BOUNDS_READ);
        infocone.setCapability(Cone.ALLOW_CHILDREN_READ);
        infocone.setPickable(true);
        infocone.setUserData("infocone#" + ip);
        Cone selectinfocone = new Cone(1.5f,1.5f,6,6,Cone.GENERATE_TEXTURE_COORDS,app.getAppearance(6));
        selectinfocone.setPickable(false);
        Switch selectinfo_switch = new Switch();
        selectinfo_switch.addChild(selectinfocone);
        selectinfo_switch.setCapability(Switch.ALLOW_SWITCH_WRITE);
        selectinfo_switch.setWhichChild(Switch.CHILD_NONE);
        
        Transform3D infotransrot = new Transform3D();
        Transform3D infotranstrans = new Transform3D();
        
        infotransrot.rotX(Math.PI/2);
        infotranstrans.setTranslation( new Vector3f(0.0f,0.0f,0.0f));
        infotranstrans.mul(infotransrot);
        infogroup.setTransform(infotranstrans);
        infogroup.addChild(infocone);
        infogroup.addChild(selectinfo_switch);
        
        this.addChild(portgroup);     
        this.addChild(infogroup);
        this.setTransform(new Transform3D());
    }
    
    public Vector3f getHostVector(){
        Vector3f return_vector =new Vector3f();
        TransformGroup temp_hostgroup= new TransformGroup();
        temp_hostgroup = (TransformGroup) this.getChild(0);
        Transform3D temp_trans = new Transform3D();
        temp_hostgroup.getTransform(temp_trans);
        temp_trans.get(return_vector);
        return return_vector;                            
    }
    public port3d getPort(int port){
        
        port3d outport = new port3d(app);
        port3d tempport = new port3d(app);
        
        TransformGroup temptrans = new TransformGroup();
        temptrans = (TransformGroup) this.getChild(1);
        
        for (int i =0 ; i< ports_list_length;i++){
            tempport = (port3d) temptrans.getChild(i);
            if(port==tempport.port){
                outport = tempport;
            }
        }
        return outport;
        
    }
    
    public void addPort(int port,boolean src) {        
      
        
	port3d newport = new port3d(port,ports_list_length, ports_list_length,host_ip,app,src);
        
        TransformGroup temptrans = new TransformGroup();
        temptrans = (TransformGroup) this.getChild(1);
        
        // Insert the Port in the right place
        boolean found=false;
        if (ports_list_length>0){            
            for (int i=0; i< ports_list_length; i++){
                port3d porttemp = new port3d(app);
                porttemp = (port3d) temptrans.getChild(i);
                if (porttemp.port > port) {
                    temptrans.insertChild(newport,i);
                    found = true;
                    break;
                }
            }
        }
        if (!found||ports_list_length<1) {
            temptrans.insertChild(newport,0);
        }
        
        ports_list_length++;       
        
        // Reposition all the ports
        
        for (int i=0; i<ports_list_length; i++){
            
            port3d porttemp = new port3d(app);
            porttemp = (port3d) temptrans.getChild(i);
            
            Transform3D porttransform_temp = new Transform3D();
            float xpos=0.0f;
            float zpos=(float) ( i * 1.5f+2.0f);
            
            porttransform_temp.set(new Vector3f(0.0f,0.0f,zpos));            
            
            TransformGroup temptrans2 = new TransformGroup();
            temptrans2 = (TransformGroup) porttemp.getChild(0);
            temptrans2.setTransform(porttransform_temp);

        }
        
        // Reposition the host to the top of the port stack
        TransformGroup temp_hostgroup= new TransformGroup();
        temp_hostgroup = (TransformGroup) this.getChild(0);
        Transform3D temp_trans = new Transform3D();
        temp_hostgroup.getTransform(temp_trans);
        float height = (1.5f * ports_list_length) +3.0f;
        temp_trans.setTranslation(new Vector3f(0.0f,0.0f,height));
        temp_hostgroup.setTransform(temp_trans);                            
    }
        
    public void detailsOn(host_stats stats_in){
        BranchGroup details_bg = new BranchGroup();
        details_bg.setCapability(BranchGroup.ALLOW_DETACH);
        TransformGroup details_tg = new TransformGroup();
        Appearance text_appear = new Appearance();
        // How big does the box need to be?
        int height = 2 + stats_in.dst_alerts.size()+stats_in.src_alerts.size();
        int width = 0;
        for (int i=0;i<stats_in.dst_alerts.size();i++){
            String cur_string = (String) stats_in.dst_alerts.elementAt(i);
            if (cur_string.length()>width){
                width = cur_string.length();
            }
        }
        for (int i=0;i<stats_in.src_alerts.size();i++){
            String cur_string = (String) stats_in.src_alerts.elementAt(i);
            if (cur_string.length()>width){
                width = cur_string.length();
            }
        }        
        
        Dimension textSize = new Dimension(nearestPower(width*10),nearestPower(height*15));
        BufferedImage text_image = new BufferedImage(textSize.width,textSize.height,BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D) text_image.getGraphics();
        g.setFont(new Font("Arial",Font.PLAIN,10));
        g.setColor(Color.black);
        g.fillRect(0,0,textSize.width,textSize.height);
        g.setColor(new Color(0.0f,0.0f,0.8f));
        g.setStroke(new BasicStroke(1.0f,BasicStroke.CAP_ROUND,
            BasicStroke.JOIN_ROUND,10.0f));
        g.fillRoundRect(0,0,textSize.width,textSize.height,30,30);
        g.setColor(Color.white);
        g.drawRoundRect(1,1,textSize.width-1,textSize.height-1,30,30);
        int srcCount = stats_in.src_alerts.size();
        int dstCount = stats_in.dst_alerts.size();
        if (srcCount!=0){

            g.drawString("Alerts as Source               Count:",20,20);
            for (int j=0;j<srcCount;j++){
                g.drawString(stats_in.src_alerts.elementAt(j) +"         " + 
                    stats_in.src_alert_counts.elementAt(j) +" ",
                    20,(j+1)*10+20);
            }
        }
        if (dstCount!=0){
            g.drawString("Alerts as Destination       Count:",20,(((srcCount+1)*10)+30));
            for (int i=0;i<dstCount;i++){
                g.drawString(stats_in.dst_alerts.elementAt(i)+"          "+
                    stats_in.dst_alert_counts.elementAt(i)+" ",
                    20,(((i+1)*10)+40+((srcCount)*10)));                
            }
        }
        g.dispose();
        Texture2D texture=new
            Texture2D(Texture2D.BASE_LEVEL,Texture2D.RGBA,textSize.width, 
                textSize.height);
        texture.setImage(0, new ImageComponent2D(ImageComponent2D.FORMAT_RGBA,text_image));
        text_appear.setTexture(texture);
        
        texture.setBoundaryModeS(Texture2D.WRAP);
        texture.setBoundaryModeT(Texture2D.WRAP);
        
        GeometryArray geom = 
            new QuadArray(
            4,
            QuadArray.COORDINATES|QuadArray.NORMALS|QuadArray.TEXTURE_COORDINATE_2);
        float scale = 15.0f;
        geom.setCoordinates(0, 
            new Point3f[]
                {
                    new Point3f(0.0f,0.0f,0.0f),
                    new Point3f(textSize.width/scale,0.0f,0.0f),
                    new Point3f(textSize.width/scale,textSize.height/scale,0.0f),
                    new Point3f(0.0f,textSize.height/scale,0.0f)
                });

        TexCoord2f[] myTexCoords = {
            new TexCoord2f(0.0f,0.0f),
            new TexCoord2f(1.0f,0.0f),
            new TexCoord2f(1.0f,1.0f),
            new TexCoord2f(0.0f,1.0f)
        };
        geom.setTextureCoordinates(0,0,myTexCoords);
        geom.setNormals(0,
            new Vector3f[] {
                new Vector3f(0,0,1),
                new Vector3f(0,0,1),
                new Vector3f(0,0,1),
                new Vector3f(0,0,1)
        });
        
        PolygonAttributes polyAttrib = new PolygonAttributes(
            PolygonAttributes.POLYGON_FILL, 
            PolygonAttributes.CULL_NONE,0.0f);
        text_appear.setPolygonAttributes(polyAttrib);
        text_appear.setTransparencyAttributes(new TransparencyAttributes(
            TransparencyAttributes.NICEST,0.5f));
        text_appear.setTextureAttributes( new TextureAttributes(
            TextureAttributes.DECAL, new Transform3D(),new Color4f(0.0f,0.0f,0.0f,1.0f),0));
        Shape3D textshape = new Shape3D(geom,text_appear);
        OrientedShape3D text_shape_oriented = new OrientedShape3D(
            textshape.getGeometry(),textshape.getAppearance(),
            OrientedShape3D.ROTATE_ABOUT_POINT,new Point3f(0.0f,0.0f,0.0f));
        details_tg.addChild(text_shape_oriented);
        Transform3D text_trans = new Transform3D();
        text_trans.setTranslation(new Vector3f(0.0f,0.0f,-8.0f));
        details_tg.setTransform(text_trans);
        details_bg.addChild(details_tg);
        this.addChild(details_bg);
        details=true;
    }
    
    public void detailsOff(){
        this.removeChild(3);
        details=false;
    }
    
    public int nearestPower(int in_value){
        int return_value=1;
        boolean found=false;
        while (!found){           
            if (return_value<in_value){
                return_value*=2;
            } else { 
                found = true;
            }
        }
        return (int) return_value;
    }
    
    public boolean isPort(int port){
       
        boolean isport = false;
                
        TransformGroup temptg = new TransformGroup();
        
        temptg = (TransformGroup) this.getChild(1);
        
        // return true for icmp packets
        if (port == -1) {
            isport = true;
            return isport;
        }
        
        for (int i =0; i< (temptg.numChildren()); i++){
            port3d outport = new port3d();
            outport = (port3d) temptg.getChild(i);            
            if (port == outport.port){
                isport=true;
	    }
        }
        
        return isport;
    }
}
