/* Scanmap is a 3d Visualisation for Snort alerts 
*   
*   Copyright (C) 2003 Daniel Clark
*
*	This program is free software; you can redistribute it and/or 
*	modify it under the terms of the GNU General Public License as 
*	published by the Free Software Foundation; either version 2 of 
*	the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful, 
*	but WITHOUT ANY WARRANTY; without even the implied warranty of 
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License 
*	along with this program; if not, write to the Free Software 
*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*	
*	Contact: Daniel Clark 
*	Email:	scanmap3d@sourceforge.net
*	
*
*
* net3d.java // Main program file
*
* Created on March 14, 2003, 12:28 PM
*/

package net.ossim.scanmap;


import java.lang.Math.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Sphere;
import java.util.*;
import java.awt.*;


public class net3d extends javax.media.j3d.TransformGroup {
    
    private net3dapp textures;
    private scanmap3d this_scanmap;
    public int type;
    public Switch host_switch;
    
    /** Create new instance of net3d */

    public net3d(int typein,net3dapp texture_in,scanmap3d in_scanmap){
    
        textures=texture_in;
        this_scanmap=in_scanmap;
        this.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.setPickable(true);
        host_switch = new Switch();
        host_switch.setCapability(Switch.ALLOW_CHILDREN_EXTEND);
        host_switch.setCapability(Switch.ALLOW_CHILDREN_READ);
        host_switch.setCapability(Switch.ALLOW_CHILDREN_WRITE);
        host_switch.setCapability(Switch.ALLOW_SWITCH_WRITE);
        host_switch.setWhichChild(Switch.CHILD_ALL);
        host_switch.setPickable(true);
        this.addChild(host_switch);
        type = typein;
        
    }    
    
    public net3d(net3dapp texture_in){
        textures=texture_in;
        type =0;
    }
    
    public void addHost(String in_ip,int type_in){      
        host3d temphost = new host3d(in_ip,textures,this_scanmap,type_in);
        BranchGroup host_bg = new BranchGroup();        
        host_bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
        host_bg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        host_bg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        host_bg.setCapability(BranchGroup.ALLOW_DETACH);
        host_bg.setPickable(true);
        host_bg.addChild(temphost);        
        host_switch.addChild(host_bg);
        
        int num_hosts = host_switch.numChildren();
        
        int height = (int) Math.sqrt((double)num_hosts/2.0);
        int width = 3 * height;
        int cur_height=0;
        int cur_width=0;
        for (int i=0; i<num_hosts; i++){
            
            Transform3D nettrans = new Transform3D();
            
            switch (this.type) {
                case 0:                    
                    nettrans.set(new Vector3f(0.0f,0.0f,0.0f));
                    break;
                case 1:
                    nettrans.set(new Vector3f(40.0f,-40.0f,0.0f));
                    break;
                case 2:
                    nettrans.set(new Vector3f(-40.0f,-40.0f,0.0f));
                    break;
                case 3:
                    nettrans.set(new Vector3f(40.0f,-80.0f,0.0f));
                    break;
                case 4:
                    nettrans.set(new Vector3f(-40.0f,-80.0f,0.0f));
                    break;
                case 5:
                    nettrans.set(new Vector3f(0.0f,120.0f,0.0f));
                    break;

            }
            
            BranchGroup temp_bg = new BranchGroup();
            host3d hosttemp = new host3d(textures);
            Transform3D hosttrans = new Transform3D();
            Transform3D hostrotate = new Transform3D();
            float xpos = (float) (cur_width-width/3) * 20.0f;
            float zpos = (float) (cur_height-height/2) * 20.0f;
            if (this.type==1){
                hostrotate.rotZ(Math.PI); 
            }
            hosttrans.set(new Vector3f(xpos,zpos,0.0f));
            hosttrans.mul(nettrans);
            hosttrans.mul(hostrotate);
            
            TransformGroup temptrans1 = new TransformGroup();

            temp_bg = (BranchGroup) host_switch.getChild(i);            
            hosttemp = (host3d) temp_bg.getChild(0);
            hosttemp.setTransform(hosttrans);
            
            cur_width++;
            if (cur_width>width){
                cur_width=0;
                cur_height++;
            }
        }
        
    };
    
    public host3d getHost(String inhost_ip){
        host3d host_req;
        host_req = new host3d(textures);
        BranchGroup temp_bg = new BranchGroup();
        
        int num_hosts = host_switch.numChildren();
        
        for (int i = 0; i<num_hosts;i++){
            temp_bg = (BranchGroup) host_switch.getChild(i);
            host_req = (host3d) temp_bg.getChild(0); 
            
            if (host_req.host_ip.equals(inhost_ip)){
                break;
            }
        }
        return host_req;
            
    }
    
    public boolean isHost(String host_ip){
        
        boolean ishost=false;        
        
        host3d host_req = new host3d(textures);        
        BranchGroup temp_bg = new BranchGroup();
        int num_hosts = host_switch.numChildren();
        
        
        
        for (int i = 0; i<num_hosts;i++){   
            temp_bg = (BranchGroup) host_switch.getChild(i);
            host_req = (host3d) temp_bg.getChild(0);
           
            if (host_req.host_ip.equals(host_ip)){
                ishost = true;                
            }            
        }
        return ishost;
            
    }
    
    public void byHostOn(){
        if (host_switch.numChildren()!=0){
            // turn the hosts on
            BitSet Host_Mask = new BitSet();
            StringBuffer strbuff = new StringBuffer();
            
            for (int i=0;i<host_switch.numChildren();i++){                
                host3d temp_host = new host3d(textures);
                BranchGroup temp_bg = (BranchGroup) host_switch.getChild(i);            
                temp_host = (host3d) temp_bg.getChild(0);
                if(temp_host.selected){
                    Host_Mask.set(i);
                }
                
                else{
                    boolean found = false;
                    for(int j=0;j<scanmap3d.view3d.links_vector.size();j++){
                        
                        ip_packet temp_link = (ip_packet) scanmap3d.view3d.links_vector.elementAt(j);
                        net3d temp_net1 = (net3d) scanmap3d.view3d.getNetwork(temp_link.dst_ip);
                        net3d temp_net2 = (net3d) scanmap3d.view3d.getNetwork(temp_link.src_ip);                        
                        host3d temp_host1 = (host3d) temp_net1.getHost(temp_link.dst_ip);
                        host3d temp_host2 = (host3d) temp_net2.getHost(temp_link.src_ip);
                        if (temp_host==temp_host1&&temp_host2.selected){
                            Host_Mask.set(i);
                            found =true;
                        }
                        else if (temp_host==temp_host2&&temp_host1.selected){
                            Host_Mask.set(i);
                            found = true;
                        }                        
                    }
                    if (!found){
                        Host_Mask.clear(i);
                    }
                }
            }
            host_switch.setWhichChild(Switch.CHILD_MASK);
            host_switch.setChildMask(Host_Mask);
        }
    }
    
    public void byHostOff(){
        host_switch.setWhichChild(Switch.CHILD_ALL);
    }

}
