/* Scanmap is a 3d Visualisation for Snort alerts
*
*   Copyright (C) 2003 Daniel Clark
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License as
*	published by the Free Software Foundation; either version 2 of
*	the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*	Contact: Daniel Clark
*	Email:	scanmap3d@sourceforge.net
*
*
*
* net3dapp.java
*
* Created on March 14, 2003, 12:28 PM
*/
package net.ossim.scanmap;


import java.awt.*;
import java.awt.image.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.image.*;

public class net3dapp {

    private Texture2D host_texture0,host_texture1,host_texture2;
    private Texture2D host_texture3,host_texture4,host_texture5;
    private Texture2D host_texture6;
    private Texture2D cone_texture;
    private Texture2D udppacket_texture,tcppacket_texture,icmppacket_texture,otherpacket_texture;
    private Texture2D blank_texture;
    private Color3f red;
    private Color3f white;
    private Color3f green;
    private Color3f blue;
    private Color3f black;
    private Color3f yellow;

//  public static final String PATH_IMAGES = "/net/ossim/scanmap/images/";
    public static final String PATH_IMAGES = "/net/ossim/scanmap/images/";


    /** Creates a new instance of net3dapp */
    public net3dapp() {
        // Load the textures
        host_texture0 = loadImage("host0.jpg");
        host_texture1 = loadImage("host1.jpg");
        host_texture2 = loadImage("host2.jpg");
        host_texture3 = loadImage("host3.jpg");
        host_texture4 = loadImage("host4.jpg");
        host_texture5 = loadImage("host5.jpg");
        host_texture6 = loadImage("host6.jpg");
        cone_texture = loadImage("cone.jpg");
        tcppacket_texture = loadImage("tcppacket.jpg");
	udppacket_texture = loadImage("udppacket.jpg");
	icmppacket_texture = loadImage("icmppacket.jpg");
	otherpacket_texture = loadImage("otherpacket.jpg");
        blank_texture = loadImage("blank.jpg");
        red = new Color3f(0.9f,0.0f,0.0f);
        white = new Color3f(1.0f,1.0f,1.0f);
        green = new Color3f(0.0f,0.9f,0.0f);
        blue = new Color3f (0.0f,0.0f,0.9f);
        yellow = new Color3f (0.0f,0.8f,0.8f);
        black = new Color3f (0.0f,0.0f,0.0f);
    }

    private Texture2D loadImage(String in_string){
        try {


        TextureLoader loader = new TextureLoader(PATH_IMAGES + in_string,null);
        ImageComponent2D image = loader.getImage();
        Texture2D  texture = new Texture2D(Texture.BASE_LEVEL,Texture.RGBA,image.getWidth(),
            image.getHeight());
        texture.setImage(0,image);
        return texture;
        }
        catch(Exception e){
            Texture2D null_texture = new Texture2D();
            return null_texture;
        }

    }


    public Appearance getAppearance(int in_texture){

        Appearance out_app = new Appearance();
        out_app.setTransparencyAttributes(new TransparencyAttributes(
            TransparencyAttributes.NICEST,0.0f));

        switch (in_texture){

	    // Various Coloured Textures

            case 1:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(0.2f,0.2f,0.2f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.5f));
                return out_app;

            case 2:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(0.2f,0.2f,0.2f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.5f));
                return out_app;

            case 3:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(0.5f,0.0f,0.5f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.5f));
                return out_app;

            case 4:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(0.5f,0.0f,0.5f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.5f));
                return out_app;

            case 5:
                out_app.setTexture(cone_texture);
                return out_app;

            case 6:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(0.5f,1.0f,0.5f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.5f));
                return out_app;

            // Packet Textures
            case 10:
                out_app.setTexture(tcppacket_texture);
                return out_app;

            case 11:
                out_app.setTexture(udppacket_texture);
                return out_app;

            case 12:
                out_app.setTexture(icmppacket_texture);
                return out_app;

            case 13:
                out_app.setTexture(otherpacket_texture);
                return out_app;


            case 20:
                out_app.setTexture(host_texture0);

                return out_app;
            case 21:
                out_app.setTexture(host_texture1);

                return out_app;
            case 22:
                out_app.setTexture(host_texture2);

                return out_app;
            case 23:
                out_app.setTexture(host_texture3);

                return out_app;
            case 24:
                out_app.setTexture(host_texture4);

                return out_app;
            case 25:
                out_app.setTexture(host_texture5);

                return out_app;
		// Port Textures
		case 30:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(.5f,0.1f,0.1f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.3f));
                return out_app;
		case 31:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(1.0f,1.0f,0.0f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.9f));
                return out_app;
		case 32:
                out_app.setColoringAttributes( new ColoringAttributes(
                    new Color3f(0.0f,0.8f,0.8f),ColoringAttributes.NICEST));
                out_app.setTransparencyAttributes(new TransparencyAttributes(
                    TransparencyAttributes.NICEST,0.5f));
                return out_app;

            default:
                out_app.setTexture(blank_texture);
                return out_app;
        }
    }








}
