/* Scanmap is a 3d Visualisation for Snort alerts 
*   
*   Copyright (C) 2003 Daniel Clark
*
*	This program is free software; you can redistribute it and/or 
*	modify it under the terms of the GNU General Public License as 
*	published by the Free Software Foundation; either version 2 of 
*	the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful, 
*	but WITHOUT ANY WARRANTY; without even the implied warranty of 
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License 
*	along with this program; if not, write to the Free Software 
*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*	
*	Contact: Daniel Clark 
*	Email:	scanmap3d@sourceforge.net
*	
*
*
* pipe3d.java
*
* Created on March 14, 2003, 12:28 PM
*/
package net.ossim.scanmap;


import java.lang.Math.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Sphere;
import java.util.*;


public class pipe3d extends javax.media.j3d.TransformGroup {
    
    /** Creates a new instance of pipe3d */
    public float length;
    
    public pipe3d(){
    }
    
    public pipe3d(Vector3f startpoint, Vector3f endpoint, float width,
        Appearance app) {
        this.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        this.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.setPickable(false);
        Vector3f cylvector = new Vector3f();
        
        cylvector.set(startpoint);
        cylvector.sub(endpoint);
        
        length = cylvector.length();
        Cylinder newcyl = new Cylinder(width,length,Cylinder.GENERATE_TEXTURE_COORDS,4,4,app);
        newcyl.setCapability(Cylinder.ALLOW_BOUNDS_READ);
        newcyl.setCapability(Cylinder.ALLOW_CHILDREN_READ);
        newcyl.setUserData("ignore");
        newcyl.setPickable(false);
        
        TransformGroup cylgroup1 = new TransformGroup();   
        cylgroup1.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        cylgroup1.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        cylgroup1.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        cylgroup1.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        
        Transform3D translatecyl1 = new Transform3D();
        translatecyl1.set(new Vector3f(0.0f,length/2.0f,0.0f));
        cylgroup1.setTransform(translatecyl1);
        cylgroup1.addChild(newcyl);
        
        Transform3D rotateY = new Transform3D();
        Transform3D rotateX = new Transform3D();
        Vector3f zaxis = new Vector3f(0.0f,0.0f,1.0f);
        Vector3f yaxis = new Vector3f(0.0f,1.0f,0.0f);
        float xangle = -yaxis.angle(cylvector);                        
        Vector3f cylvectorzx = new Vector3f(-cylvector.x,0.0f,-cylvector.z);
        float yangle;
        if (cylvector.x==0 && cylvector.z==0){
            yangle=0;
        }
        else { 
            yangle = zaxis.angle(cylvectorzx);
        }
        float pifloat = (float) Math.PI;
        
        if (cylvector.x>0) {
            yangle=-yangle;
        }
        if (xangle==0){
            yangle=0;
        }
        
        rotateY.rotY((double)yangle);
        rotateX.rotX((double)xangle);
        
        TransformGroup cylgroup2 = new TransformGroup();        
        Transform3D translatecyl2 = new Transform3D();
        translatecyl2.set(endpoint);
        translatecyl2.mul(rotateY);
        translatecyl2.mul(rotateX);

        cylgroup2.setTransform(translatecyl2);
        cylgroup2.addChild(cylgroup1);
        cylgroup2.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
        cylgroup2.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
        cylgroup2.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
        cylgroup2.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        cylgroup2.setPickable(false);
        this.addChild(cylgroup2);
    }
    
    public void updatePipe(Vector3f startpoint, Vector3f endpoint, float width,
        Appearance app){
            
            
            
        Vector3f cylvector = new Vector3f();
        
        cylvector.set(startpoint);
        cylvector.sub(endpoint);
        
        float cyllength = cylvector.length();

        float scale = cyllength/this.length;
        
        TransformGroup cylgroup1 = new TransformGroup();
        TransformGroup cylgroup2 = new TransformGroup();  
        cylgroup2 = (TransformGroup) this.getChild(0);
        cylgroup1 = (TransformGroup) cylgroup2.getChild(0);
        
        Transform3D scaletrans = new Transform3D();
        scaletrans.setScale(new Vector3d(1.0f,scale,1.0f));
        
        Transform3D translatecyl1 = new Transform3D();
        translatecyl1.set(new Vector3f(0.0f,cyllength/2.0f,0.0f));
        translatecyl1.mul(scaletrans);
        cylgroup1.setTransform(translatecyl1);
        
        Transform3D rotateY = new Transform3D();
        Transform3D rotateX = new Transform3D();
        Vector3f zaxis = new Vector3f(0.0f,0.0f,1.0f);
        Vector3f yaxis = new Vector3f(0.0f,1.0f,0.0f);
        float xangle = -yaxis.angle(cylvector);                        
        Vector3f cylvectorzx = new Vector3f(-cylvector.x,0.0f,-cylvector.z);
        float yangle = zaxis.angle(cylvectorzx);
        float pifloat = (float) Math.PI;
        
        if (cylvector.x==0 && cylvector.z==0){
            yangle=0;
        }
        else { 
            yangle = zaxis.angle(cylvectorzx);
        }        
        
        if (cylvector.x>0) {
            yangle=-yangle;
        }
        if (xangle==0){
            yangle=0;
        }

        
        rotateY.rotY((double)yangle);
        rotateX.rotX((double)xangle);
      
        Transform3D translatecyl2 = new Transform3D();
        
        translatecyl2.set(endpoint);

        translatecyl2.mul(rotateY);
        translatecyl2.mul(rotateX);


        cylgroup2.setTransform(translatecyl2);                
            
    }
    
}
